<?php
/**
 * A general Handler for raw data from sockets
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseRawDataHandler extends BaseHandler {
	// Error codes:
	// - Socket raw data stream errors
	const SOCKET_ERROR_UNKNOWN                    = 'unknown_error';              // Unknown error (should not happen)
	const SOCKET_ERROR_TRANSPORT_ENDPOINT         = 'transport_endpoint';         // Transport endpoint has closed
	const SOCKET_ERROR_INVALID_BASE64_MODULO      = 'base64_modulo';              // Length is not modulo 4
	const SOCKET_ERROR_INVALID_BASE64_MESSAGE     = 'base64_message';             // Raw data is not Base64-encoded
	const SOCKET_ERROR_UNHANDLED                  = 'unhandled_package';          // Unhandled raw data (not bad)
	const SOCKET_ERROR_CONNECTION_REFUSED         = 'connection_refused';         // The name says it: connection refused
	const SOCKET_ERROR_CONNECTION_TIMED_OUT       = 'connection_timed_out';       // The name says it: connection attempt has timed-out
	const SOCKET_ERROR_OPERATION_IN_PROGRESS      = 'operation_in_progress';      // 'Operation now in progress'
	const SOCKET_ERROR_OPERATION_ALREADY_PROGRESS = 'operation_already_progress'; // 'Operation already in progress'
	const SOCKET_ERROR_RESOURCE_UNAVAILABLE       = 'resource_unavailable';       // 'Resource temporary unavailable'
	const SOCKET_ERROR_NO_ROUTE_TO_HOST           = 'no_route_to_host';           // The name says it: no route to host
	const SOCKET_ERROR_CONNECTION_RESET_BY_PEER   = 'connection_reset_by_peer';   // Connection reset by peer
	const SOCKET_ERROR_BROKEN_PIPE                = 'broken_pipe';                // Broken pipe
	const SOCKET_CONNECTED                        = 'connected';                  // Nothing errorous happens, socket is connected

	// - Package errors
	const PACKAGE_ERROR_INVALID_DATA       = 'invalid_data';    // Invalid data in package found
	const PACKAGE_ERROR_INCOMPLETE_DATA    = 'incomplete_data'; // Incomplete data sent (e.g. field is missing)
	const PACKAGE_ERROR_INVALID_CONTENT    = 'invalid_content'; // Content is invalid (e.g. not well-formed)
	const PACKAGE_ERROR_RECIPIENT_MISMATCH = 'recipient_error'; // Recipient is not us
	const PACKAGE_LEVEL_CHECK_OKAY         = 'checked_package'; // Package is fine

	// Package data
	const PACKAGE_RAW_DATA   = 'raw_data';
	const PACKAGE_ERROR_CODE = 'error_code';

	// Start/end marker
	const STREAM_START_MARKER = '[[S]]';
	const STREAM_END_MARKER   = '[[E]]';

	/**
	 * Stacker for raw data
	 */
	const STACKER_NAME_RAW_DATA = 'raw_data';

	/**
	 * Error code from socket
	 */
	private $errorCode = -1;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Set error code to 'unknown'
		$this->setErrorCode(self::SOCKET_ERROR_UNKNOWN);

		// Init stacker instance for processed raw data
		$stackInstance = ObjectFactory::createObjectByConfiguredName('node_raw_data_stacker_class');

		// Remember this in this package handler
		$this->setStackInstance($stackInstance);

		// Init stacker
		$this->initStack();
	}

	/**
	 * Initializes the stacker for raw data
	 *
	 * @return	void
	 */
	protected function initStack () {
		$this->getStackInstance()->initStack(self::STACKER_NAME_RAW_DATA);
	}

	/**
	 * Adds given raw data to the raw data stacker
	 *
	 * @param	$rawData	raw data from the socket resource
	 * @return	void
	 */
	protected function addRawDataToStacker ($rawData) {
		/*
		 * Add the deocoded data and error code to the stacker so other classes
		 * (e.g. NetworkPackage) can "pop" it from the stacker.
		 */
		$this->getStackInstance()->pushNamed(self::STACKER_NAME_RAW_DATA, array(
			self::PACKAGE_RAW_DATA   => $rawData,
			self::PACKAGE_ERROR_CODE => $this->getErrorCode()
		));
	}

	/**
	 * Checks whether raw data is pending for further processing.
	 *
	 * @return	$isPending	Whether raw data is pending
	 */
	public function isRawDataPending () {
		// Does the stacker have some entries (not empty)?
		$isPending = (!$this->getStackInstance()->isStackEmpty(self::STACKER_NAME_RAW_DATA));

		// Return it
		return $isPending;
	}

	/**
	 * "Getter" for next raw data from the stacker
	 *
	 * @return	$rawData	Raw data from the stacker
	 */
	public function getNextRawData () {
		// "Pop" the raw data from the stacker
		$rawData = $this->getStackInstance()->popNamed(self::STACKER_NAME_RAW_DATA);

		// And return it
		return $rawData;
	}

	/**
	 * Checks whether the 'recipient' field matches our own an universal node
	 * locator.
	 *
	 * @param	$packageData	Raw package data
	 * @return	$matches		Whether it matches
	 * @todo	This method will be moved to a better place
	 */
	protected function ifRecipientMatchesOwnUniversalNodeLocator (array $packageData) {
		// Construct own address first
		$ownAddress = NodeObjectFactory::createNodeInstance()->determineUniversalNodeLocator();

		// Does it match?
		$matches = ($ownAddress === $packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT]);

		// Return result
		return $matches;
	}

	/**
	 * Setter for error code
	 *
	 * @param	$errorCode	The error code we shall set
	 * @return	void
	 */
	public final function setErrorCode ($errorCode) {
		$this->errorCode = $errorCode;
	}

	/**
	 * Getter for error code
	 *
	 * @return	$errorCode	The error code
	 */
	public final function getErrorCode () {
		return $this->errorCode;
	}
}

// [EOF]
?>
