<?php
/**
 * A generic XML template engine class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 * @todo		This template engine does not make use of setTemplateType()
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseXmlTemplateEngine extends BaseTemplateEngine {
	/**
	 * Main nodes in the XML tree
	 */
	private $mainNodes = array();

	/**
	 * Sub nodes in the XML tree
	 */
	private $subNodes = array();

	/**
	 * Current main node
	 */
	private $curr = array();

	/**
	 * XML template type
	 */
	private $xmlTemplateType = 'xml';

	/**
	 * Type prefix
	 */
	private $typePrefix = 'xml';

	/**
	 * Name of stacker
	 */
	private $stackerName = '';

	/**
	 * Content from dependency
	 */
	protected $dependencyContent = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Does a generic initialization of the template engine
	 *
	 * @param	$typePrefix				Type prefix
	 * @param	$xmlTemplateType		Type of XML template
	 * @return	$templateInstance		An instance of TemplateEngine
	 * @throws	BasePathIsEmptyException		If the provided $templateBasePath is empty
	 * @throws	InvalidBasePathStringException	If $templateBasePath is no string
	 * @throws	BasePathIsNoDirectoryException	If $templateBasePath is no
	 *											directory or not found
	 * @throws	BasePathReadProtectedException	If $templateBasePath is
	 *											read-protected
	 */
	protected function initXmlTemplateEngine ($typePrefix, $xmlTemplateType) {
		// Get template instance
		$applicationInstance = Registry::getRegistry()->getInstance('app');

		// Determine base path
		$templateBasePath = $this->getConfigInstance()->getConfigEntry('application_base_path') . $applicationInstance->getRequestInstance()->getRequestElement('app') . '/';

		// Is the base path valid?
		if (empty($templateBasePath)) {
			// Base path is empty
			throw new BasePathIsEmptyException($this, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!is_string($templateBasePath)) {
			// Is not a string
			throw new InvalidBasePathStringException(array($this, $templateBasePath), self::EXCEPTION_INVALID_STRING);
		} elseif (!is_dir($templateBasePath)) {
			// Is not a path
			throw new BasePathIsNoDirectoryException(array($this, $templateBasePath), self::EXCEPTION_INVALID_PATH_NAME);
		} elseif (!is_readable($templateBasePath)) {
			// Is not readable
			throw new BasePathReadProtectedException(array($this, $templateBasePath), self::EXCEPTION_READ_PROTECED_PATH);
		}

		// Set the base path
		$this->setTemplateBasePath($templateBasePath);

		// Set template extensions
		$this->setRawTemplateExtension($this->getConfigInstance()->getConfigEntry('raw_template_extension'));
		$this->setCodeTemplateExtension($this->getConfigInstance()->getConfigEntry($typePrefix . '_message_template_extension'));

		// Absolute output path for compiled templates
		$this->setCompileOutputPath($this->getConfigInstance()->getConfigEntry('base_path') . $this->getConfigInstance()->getConfigEntry('compile_output_path'));

		// Init a variable stacker
		$stackInstance = ObjectFactory::createObjectByConfiguredName($typePrefix . '_' . $xmlTemplateType . '_stacker_class');

		// Set name
		$this->stackerName = $typePrefix . '_' . $xmlTemplateType;

		// Init stacker
		$stackInstance->initStack($this->stackerName);

		// Set it
		$this->setStackInstance($stackInstance);

		// Set XML template type and prefix
		$this->xmlTemplateType = $xmlTemplateType;
		$this->typePrefix      = $typePrefix;

		// Set it in main nodes
		array_push($this->mainNodes, str_replace('_', '-', $xmlTemplateType));
	}

	/**
	 * Load a specified XML template into the engine
	 *
	 * @param	$templateName	Optional name of template
	 * @return	void
	 */
	public function loadXmlTemplate ($templateName = '') {
		// Is the template name empty?
		if (empty($templateName)) {
			// Set generic template name
			$templateName = $this->typePrefix . '_' . $this->xmlTemplateType . '_template_type';
		} // END - if

		// Set template type
		$this->setTemplateType($this->getConfigInstance()->getConfigEntry($templateName));

		// Load the special template
		$this->loadTemplate($this->xmlTemplateType);
	}

	/**
	 * Getter for current main node
	 *
	 * @return	$currMainNode	Current main node
	 */
	public final function getCurrMainNode () {
		return $this->curr['main_node'];
	}

	/**
	 * Setter for current main node
	 *
	 * @param	$element		Element name to set as current main node
	 * @return	$currMainNode	Current main node
	 */
	private final function setCurrMainNode ($element) {
		$this->curr['main_node'] = (string) $element;
	}

	/**
	 * Getter for main node array
	 *
	 * @return	$mainNodes	Array with valid main node names
	 */
	public final function getMainNodes () {
		return $this->mainNodes;
	}

	/**
	 * Getter for stacker name
	 *
	 * @return	$stackerName	Name of stacker of this class
	 */
	protected final function getStackerName () {
		return $this->stackerName;
	}

	/**
	 * Setter for sub node array
	 *
	 * @param	$subNodes	Array with valid sub node names
	 * @return	void
	 */
	public final function setSubNodes (array $subNodes) {
		$this->subNodes = $subNodes;
	}

	/**
	 * Getter for sub node array
	 *
	 * @return	$subNodes	Array with valid sub node names
	 */
	public final function getSubNodes () {
		return $this->subNodes;
	}

	/**
	 * Read XML variables by calling readVariable() with 'general' as
	 * variable stack.
	 *
	 * @param	$key	Key to read from
	 * @return	$value	Value from variable
	 */
	public function readXmlData ($key) {
		// Read the variable
		$value = parent::readVariable($key, 'general');

		// Is this null?
		if (is_null($value)) {
			// Bah, needs fixing.
			$this->debugInstance('key=' . $key . ' returns NULL');
		} // END - if

		// Return value
		return $value;
	}

	/**
	 * Handles the template dependency for given XML node
	 *
	 * @param	$node					The XML node we should load a dependency template
	 * @param	$templateDependency		A template to load to satisfy dependencies
	 * @return	void
	 */
	protected function handleTemplateDependency ($node, $templateDependency) {
		// Check that the XML node is not empty
		assert(!empty($node));

		// Is the template dependency set?
		if ((!empty($templateDependency)) && (!isset($this->dependencyContent[$node]))) {
			// Get a temporay template instance
			$templateInstance = XmlTemplateEngineFactory::createXmlTemplateEngineInstance($this->typePrefix . '_' . $this->convertDashesToUnderscores($node) . '_' . $this->xmlTemplateType . '_template_class');

			// Then load it
			$templateInstance->loadXmlTemplate($templateDependency);

			// Parse the XML content
			$templateInstance->renderXmlContent();

			// Save the parsed raw content in our dependency array
			$this->dependencyContent[$node] = $templateInstance->getRawTemplateData();
		} // END - if
	}

	/**
	 * Handles the start element of an XML resource
	 *
	 * @param	$resource		XML parser resource (currently ignored)
	 * @param	$element		The element we shall handle
	 * @param	$attributes		All attributes
	 * @return	void
	 * @throws	InvalidXmlNodeException		If an unknown/invalid XML node name was found
	 */
	public final function startElement ($resource, $element, array $attributes) {
		// Initial method name which will never be called...
		$methodName = 'init' . $this->convertToClassName($this->xmlTemplateType);

		// Make the element name lower-case
		$element = strtolower($element);

		// Is the element a main node?
		//* DEBUG: */ echo "START: &gt;".$element."&lt;<br />\n";
		if (in_array($element, $this->getMainNodes())) {
			// Okay, main node found!
			$methodName = 'start' . $this->convertToClassName($element);

			// Set it
			$this->setCurrMainNode($element);
		} elseif (in_array($element, $this->getSubNodes())) {
			// Sub node found
			$methodName = 'start' . $this->convertToClassName($element);
		} else {
			// Invalid node name found
			throw new InvalidXmlNodeException(array($this, $element, $attributes), XmlParser::EXCEPTION_XML_NODE_UNKNOWN);
		}

		// Call method
		call_user_func_array(array($this, $methodName), $attributes);
	}

	/**
	 * Ends the main or sub node by sending out the gathered data
	 *
	 * @param	$resource	An XML resource pointer (currently ignored)
	 * @param	$nodeName	Name of the node we want to finish
	 * @return	void
	 * @throws	XmlNodeMismatchException	If current main node mismatches the closing one
	 */
	public final function finishElement ($resource, $nodeName) {
		// Make all lower-case
		$nodeName = strtolower($nodeName);

		// Does this match with current main node?
		//* DEBUG: */ echo "END: &gt;".$nodeName."&lt;<br />\n";
		if (($nodeName != $this->getCurrMainNode()) && (in_array($nodeName, $this->getMainNodes()))) {
			// Did not match!
			throw new XmlNodeMismatchException (array($this, $nodeName, $this->getCurrMainNode()), XmlParser::EXCEPTION_XML_NODE_MISMATCH);
		} // END - if

		// Construct method name
		$methodName = 'finish' . $this->convertToClassName($nodeName);

		// Call the corresponding method
		//* DEBUG: */ echo "call: ".$methodName."<br />\n";
		call_user_func_array(array($this, $methodName), array());
	}
}

// [EOF]
?>
